header
{
/**
 * Parser for the Mile language.
 * 
 * @author Ricardo Soto
 * @since 1.5
 */

package cl.ucv.inf.milecompiler.compilers;

}
//======================================================
// Object allowing syntactic analysis

class MileParser extends Parser;
options
{
    k = 2;
    importVocab = MileLexerVocab;
    exportVocab = MileParserVocab;
    buildAST = true;
    defaultErrorHandler=false;
}


tokens
{
//=======================================================
// Imaginary Tokens 


    PROGRAM;
    VAR_DEC;
    UN_MINUS;
    BODY;
    ASSIGN;
    FOR_ST;
    IF_ST;
    ELSE_ST;
    FOR_HEADER;

}

//======================================================= 
//Main rule

program : VAR_RW! var_dec BEGIN_RW! body END_RW!
          {## = #( #[PROGRAM, "PROGRAM"] ,##);}
        ;

//=======================================================
// Var dec

var_dec : (type IDENT SEMICOLON!)* 
          {## = #( #[VAR_DEC, "VAR_DEC"] ,##);};

type:     NUMERIC_TYPE|STRING_TYPE;

//=======================================================
// Assignment

assign : IDENT ASSIG expr SEMICOLON!
        {## = #( #[ASSIGN, "ASSIGN"] ,##);};
        
//=======================================================
// Body

body : (assign | st | print | read)*
       {## = #( #[BODY, "BODY"] ,##);};

st : (for_st|if_st);

//=======================================================
// For

for_st : FOR_RW   for_header LEFT_BRACE! body RIGHT_BRACE!
        {## = #( #[FOR_ST, "FOR_ST"] ,##);};

for_header : LEFT_PAREN! for_init for_cond SEMICOLON! INT_LITERAL RIGHT_PAREN!
             {## = #( #[FOR_HEADER, "FOR_HEADER"] ,##);};

for_init : assign;

for_cond : expr;


//=======================================================
// If

if_st : IF_RW  LEFT_PAREN! if_cond RIGHT_PAREN! LEFT_BRACE! body RIGHT_BRACE!  (else_st)?
        {## = #( #[IF_ST, "IF_ST"] ,##);};
        
if_cond : expr;
 

else_st: ELSE_RW  LEFT_BRACE! body RIGHT_BRACE!
        {## = #( #[ELSE_ST, "ELSE_ST"] ,##);};
 
//=======================================================
//Expressions 


expr          :  expAND  (OR_RW^ expAND)*;

expAND        :  expCons (AND_RW^ expCons)*;

expCons       :  expSum  ((EQUAL^| 
                           LE_EQUAL^|
                           GR_EQUAL^|
                           NOT_EQUAL^|
                           GR^|
                           LE^) expSum)* ; 

expSum        :  expProduct  ((PLUS^| 
                               SUB^) expProduct)* ; 

expProduct    :  expMinus ((STAR^ | 
                            SLASH^) expMinus)* ;  

expMinus      : (SUB! expNot)
                { ##=#(#[UN_MINUS, "UN_MINUS"], ##) ;} 
              | ((PLUS!)? expNot)
              ;

expNot: (NOT_RW^)? (IDENT|lit|(LEFT_PAREN! expr RIGHT_PAREN!));


lit: REAL_LITERAL|INT_LITERAL;



//=======================================================
// in-out

read : READ_RW LEFT_PAREN! IDENT RIGHT_PAREN! SEMICOLON! ;

print : PRINT_RW LEFT_PAREN! STRING_LITERAL COMMA! IDENT RIGHT_PAREN! SEMICOLON! ;  


